import { Box, Button, Chip, CircularProgress, Popover, Stack, useTheme } from "@mui/material";
import { fetch } from "@tauri-apps/plugin-http";
import { useAtom } from "jotai";
import { MouseEvent, useState } from "react";
import { serverConnectionAtom, serverUrlAtom } from "../../lib/jotai";
import { defaultSettings } from "../../lib/settings";
import { ServerUrlInput } from "../Home/ServerUrlInput";

export const ServerStatus = () => {
  // contexts
  const theme = useTheme();

  // atoms
  const [serverConnection, setServerConnection] = useAtom(serverConnectionAtom);
  const [serverUrl, setServerUrl] = useAtom(serverUrlAtom);

  // use states
  const [anchorEl, setAnchorEl] = useState<HTMLButtonElement | null>(null);

  const clickEvent = (event: MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget);
  };

  const dialogCloseEvent = () => {
    setAnchorEl(null);
  };

  // function to check server health
  const checkServerConnection = async () => {
    fetch(serverUrl + "/health")
      .then((response) => {
        if (response.ok) {
          setServerConnection("connected");
        } else {
          setServerConnection("disconnected");
        }
      })
      .catch(() => {
        setServerConnection("disconnected");
      });

    // if the server is connected then continue to ping every 5 seconds
    if (serverConnection === "connected") {
      setTimeout(checkServerConnection, 5000);
    }
  };

  return (
    <Box
      sx={{
        alignItems: "center",
        display: "flex",
        flexDirection: "row",
        p: 1,
      }}
    >
      {serverConnection === "connecting" && (
        <CircularProgress
          size='20px'
          sx={{ mr: 1 }}
        />
      )}
      <Chip
        color='error'
        label='Server disconnected'
        onClick={clickEvent}
        size='small'
      />
      <Popover
        anchorEl={anchorEl}
        anchorOrigin={{
          vertical: "bottom",
          horizontal: "center",
        }}
        onClose={dialogCloseEvent}
        open={Boolean(anchorEl)}
        transformOrigin={{
          vertical: "top",
          horizontal: "center",
        }}
        sx={{
          "transform": "translate(0px, 4px)",
          "& .MuiPaper-root": {
            borderRadius: defaultSettings.style.radius + "px",
          },
        }}
      >
        <Stack
          alignItems='center'
          display='flex'
          direction='row'
          spacing={1}
          sx={{
            backgroundColor: "#434446",
            p: 2,
          }}
        >
          <form
            onSubmit={() => {
              checkServerConnection();
            }}
          >
            <ServerUrlInput />
            <Button
              type='submit'
              variant='outlined'
            >
              Connect
            </Button>
          </form>
        </Stack>
      </Popover>
    </Box>
  );
};
